/*********************************************************************
**  Device:     A9101-F4
**  File:		main.c
**  Target:		
**  Tools:		ICE
**  Updated:	2017-10-12
**  Description: A9101 A90 Reference code FIFO mode 
**  This file is a sample code for your reference.
**
**  Copyright (C) 2017 AMICCOM Corp.
**
*********************************************************************/

#include "..\include\define.h"
#include "..\include\A9101.h"
#include "..\include\A9101Config.h"
#include "..\include\Uti.h"

/*********************************************************************
**  I/O Declaration
*********************************************************************/
#define CKO 		P1_3 		//CKO
#define GIO1 		P0_7 		//GIO1
#define GIO2 		P1_2 		//GIO2

/*********************************************************************
**  Constant Declaration
*********************************************************************/
#define TIMEOUT     		50				//50ms
#define t0hrel      		1366			//1ms @16.384MHz
#define BaudRate_9600		(65536-27)		//crystal=16.384MHz, timer1
#define Enable_RFINT_TMR	0x01
#define Enable_RFINT_WTR	0x02
#define Enable_RFINT_TWWS	0x04
#define Enable_RFINT_ADC    0x08
#define Enable_RFINT_EOP    0x10
#define Enable_RFINT_FPF	0x20
#define Enable_RFINT_FSTNC	0x80

/*********************************************************************
**  Global Variable Declaration
*********************************************************************/
extern const Uint8 code A9101Config_2M[];
extern const Uint8 code A9101Config_512K[];

Uint8   data    timer;
Uint8   data    TimeoutFlag;
Uint16  idata   RxCnt;
Uint32  idata   Err_ByteCnt;
Uint32  idata   Err_BitCnt;
Uint16  idata   TimerCnt0;
Uint8   xdata   A9101Config[109];
Uint8   data    *Uartptr;
Uint8   data    UartSendCnt;
Uint8   data    CmdBuf[12];
Uint8   xdata   tmpbuf[64];
Uint8   data    Flag_MASTER;
Uint8	data	RF_FLAG;
Uint8 	data 	RSSI_value;

const Uint8 code BitCount_Tab[16]={0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4};
const Uint8 code ID_Tab[8]={0x34,0x75,0xC5,0x8C,0xC7,0x33,0x45,0xE7};   //ID code
const Uint8 code PN9_Tab[]=
{   
	0xFF,0x83,0xDF,0x17,0x32,0x09,0x4E,0xD1,
    0xE7,0xCD,0x8A,0x91,0xC6,0xD5,0xC4,0xC4,
    0x40,0x21,0x18,0x4E,0x55,0x86,0xF4,0xDC,
    0x8A,0x15,0xA7,0xEC,0x92,0xDF,0x93,0x53,
    0x30,0x18,0xCA,0x34,0xBF,0xA2,0xC7,0x59,
    0x67,0x8F,0xBA,0x0D,0x6D,0xD8,0x2D,0x7D,
    0x54,0x0A,0x57,0x97,0x70,0x39,0xD2,0x7A,
    0xEA,0x24,0x33,0x85,0xED,0x9A,0x1D,0xE0
};	// This table are 64bytes PN9 pseudo random code.



/*********************************************************************
**  function Declaration
*********************************************************************/
void InitMCU(void);
void InitTimer0(void);
void InitUART0_Timer1(void);
void A9101_Config(void);
void A9101_WriteID(void);
void ResetCMD(Uint8);
void StrobeCMD(Uint8);
void SetRFCH(Uint8);
void A9101_WriteReg(Uint16, Uint8);
Uint8 A9101_ReadReg(Uint16);
void A9101_WriteFIFO(void);
void RxPacket(void);
void Err_State(void);
void Timer0ISR(void);
void UART0ISR(void);
void RFISR(void);
void TMR_enable(void);
void PM1(void);
void PM2(void);
void PM3(void);
void RCOSC_Cal(void);
void RTC_Enable(void);
void SAR_ADC_12B(void);
void RSSI_measurement(void);
void WOR_enable_by_preamble(void);
void WOR_enable_by_sync(void);
void WOR_enable_by_carrier(void);
void WOT_enable(void);
void TWOR_enable(void);
void A9101_Config(void);
void Auto_Resend(void);
void Auto_ACK(void);
void FCC(void);
extern void InitRF(void);
/*********************************************************************
* main loop
*********************************************************************/
void main(void)
{

	PCON &= ~0x10;		//PWE=0

    InitMCU();
    InitTimer0();
	InitUART0_Timer1();
    TR0=1; 				//Timer0 on
    EA=1; 				//enable interrupt

	Delay100us(5);		//delay 500us for regulator stabilized
  	ResetCMD(RF_RST);	//reset A9101 chip

	InitRF();			//init RF
						//XCL=56, (EXT1, VCOSC[5:0]=56)
	SetRFCH(60);		//915.361MHz

    ERFINT=1;			//enable RF interrupt
    A9101_WriteReg(INTSW_REG, Enable_RFINT_WTR);	//enable WTR interrupt

	//User define
	//A9101_WriteReg(FIFO1_REG, 0x3F);	//FIFO Length 64Byte	
	//A9101_WriteReg(TXTEST_REG, 0x37);	//TX Power 7, 2, 1
	//A9101_WriteReg(EXT1_REG, 0x78);	//XCL=56, (EXT1, VCOSC[5:0]=56)

    if(P3_5)//P3_5=1, master
		Flag_MASTER = 1;
    else	//P3_5=0, slave
		Flag_MASTER = 0;

//master
    if(Flag_MASTER)
    {				    
		A9101_WriteReg(GIO1_REG, 0x3C);		//GPIO1=FMTDO
		A9101_WriteReg(GIO2_REG, 0x00);		//GPIO2=WTR
		A9101_WriteReg(CKO_REG, 0x00);		//CKO=DCK	

        while(1)
        {
			A9101_WriteReg(GIO1_REG, 0x3C);		//GPIO1=FMTDO
			A9101_WriteFIFO();		//write 64 Byte data to tx fifo
			StrobeCMD(CMD_TX);

            RF_FLAG=1;
            while(RF_FLAG);		//wait transmit completed
            Delay100us(2);		//wait TX ramp down
	
//			StrobeCMD(CMD_STBY);
//			A9101_WriteReg(GIO1_REG, 0x20);	//GPIO1=RXD
//     		RF_FLAG = 1;

//			StrobeCMD(CMD_RX);
//			timer = 0;
//			TimeoutFlag = 0;			
//      		while((RF_FLAG == 1) && (TimeoutFlag == 0));	//wait receive completed
//      		if( TimeoutFlag )
//			{
//        		StrobeCMD (CMD_STBY);
//			}
//      		else
//      		{
//        		RxPacket();		//read 64 Byte data from fifo
//        		Delay1ms(10);
//			}

//			StrobeCMD(CMD_STBY); 
        }

    }
//========================================================================================
//Slave
    else
    {
	    A9101_WriteReg(GIO1_REG, 0x20);		//GPIO1=RXD
		A9101_WriteReg(GIO2_REG, 0x00);		//GPIO2=WTR
		A9101_WriteReg(CKO_REG, 0x08);		//CKO=RCK		
		
		RSSI_value=0;
		
        while(1)
        {	 
			RSSI_measurement();

//			A9101_WriteReg(GIO1_REG, 0x20);	//GPIO1=RXD	  		
//			StrobeCMD(CMD_RX);		
//  		    RF_FLAG=1;
//            while(RF_FLAG);		//wait receive completed  
//            RxPacket();			//read 64 Byte data from fifo	
//			
//			A9101_WriteReg(GIO1_REG, 0x3C);	//GPIO1=FMTDO		
//			A9101_WriteFIFO(); 	//write 64 Byte data to tx fifo
//			StrobeCMD(CMD_TX);
//			RF_FLAG=1;
//        	while(RF_FLAG);		//wait transmit completed
//       		Delay100us(2);		//wait TX ramp down				
//			StrobeCMD(CMD_STBY);
       
		}
    }

}

/************************************************************************
**  Timer0ISR
************************************************************************/
void Timer0ISR(void) interrupt 1
{
    TH0 = (65536-t0hrel)>>8;	//Reload Timer0 high byte, low byte
    TL0 = 65536-t0hrel;
	TF0 = 0;

    timer++;
    if(timer >= TIMEOUT)
    {
        TimeoutFlag=1;
    }

    TimerCnt0++;
    if(TimerCnt0 == 500)
    {
        TimerCnt0=0;
        CmdBuf[0]=0xF1;

        memcpy(&CmdBuf[1], &RxCnt, 2);
        memcpy(&CmdBuf[3], &Err_ByteCnt, 4);
        memcpy(&CmdBuf[7], &Err_BitCnt, 4);
		CmdBuf[11]=RSSI_value;

        UartSendCnt=12;
		Uartptr=&CmdBuf[0];
        SBUF=CmdBuf[0];
    }
}

/************************************************************************
**  UART0ISR
************************************************************************/
void UART0ISR(void) interrupt 4 using 3
{
	if ( TI )
	{
		TI = 0;
		UartSendCnt--;
		if ( UartSendCnt != 0 )
		{
			Uartptr++;
			SBUF = *Uartptr;
		}
	}
	
	if ( RI )
	{
		RI = 0;
	}
}

/************************************************************************
**  RFISR
************************************************************************/
void RFISR(void) interrupt 10
{
	EIF |= 0x08;	//clear flag
	RF_FLAG = 0;

	//A9101_WriteReg(TMRCTL_REG, 0xE1);	//TMRCKS=128Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE3);	//TMRCKS=64Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE5);	//TMRCKS=32Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE7);	//TMRCKS=16Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE9);	//TMRCKS=8Hz
	//A9101_WriteReg(TMRCTL_REG, 0xEB);	//TMRCKS=4Hz
	//A9101_WriteReg(TMRCTL_REG, 0xED);	//TMRCKS=2Hz
	//A9101_WriteReg(TMRCTL_REG, 0xEF);	//TMRCKS=1Hz
	


}

/************************************************************************
**  InitMCU
************************************************************************/
void InitMCU(void)
{
    P0		= 0xFF;
	P0OE	= 0xFF;
	P0PUN	= 0xFF;
	P0WUN	= 0xFF;

    P1 		= 0xFF;
	P1OE	= 0xFF;
	P1PUN	= 0xFF;
	P1WUN	= 0xFF;

	P2 		= 0xFF;
	P2OE	= 0xFF;
	P2PUN	= 0xFF;
	P2WUN	= 0xFF;

    P3 		= 0xFF;
	P3OE	= 0xDE;		//P3.0=RXD(input), P3.5=Mode Select(input)
	P3PUN	= 0xDE;
	P3WUN	= 0xFF;
	
	IOSEL 	= IOSEL | 0x08;		//enable RF GPIO pin : GIO1=P0.7, GIO2=P1.2, CKO=P1.3
}

/************************************************************************
**  InitTimer0
************************************************************************/
void InitTimer0(void)
{
    TR0 	= 0;
    TMOD	= (TMOD & 0xF0) | 0x01;	//Timer0 mode1
    TH0 	= (65536-t0hrel)>>8;	//setup Timer0 high byte, low byte
    TL0 	= 65536-t0hrel;
    TF0 	= 0;					//Clear any pending Timer0 interrupts
    ET0 	= 1;					//Enable Timer0 interrupt
}

/************************************************************************
**  InitUART0_Timer1
************************************************************************/
void InitUART0_Timer1(void)
{
//	IOSEL 	= IOSEL | 0x01; 		//enable UART pin

//	TR1		= 0;
//	TMOD	= (TMOD & 0x0F) | 0x20;	//Timer1 mode2
//	TH1 	= (256 - BaudRate_9600);//Baud Rate
//    TL1 	= (256 - BaudRate_9600);//Baud Rate
//	PCON	= 0x80;					//SMOD=1
//	CKCON	= 0x10;					//T1M=1
//    SCON	= 0x40;					//UATR0 mode1 : 8bits UART
//	T2CON	= 0x00;					//UART0 use Timer1
//	REN		= 1;
//	TR1		= 1;
//    ES		= 1;					//enable uart0 interrupt
	
	IOSEL 	= IOSEL | 0x01; 		//enable UART pin

	TR1		= 0;
	TMOD	= (TMOD & 0x0F) | 0x20;	//Timer1 mode2
	TH1 	= (256 - 27);	//Baud Rate 9600
	TL1 	= (256 - 27);	//Baud Rate
	PCON	= 0x80;					//SMOD=1 //SMOD= 0 T1ov/32
											 //SMOD= 1 T1ov/16
	CKCON	= 0x10;					//T1M=0 system clock/4
    SCON	= 0x40;					//UATR0 mode1 : 8bits UART  	
	T2CON	= 0x00;					//UART0 use Timer1
	REN		= 1;
	TR1		= 1;
    ES		= 1;					//enable uart0 interrupt
}

/************************************************************************
**  A9101_WriteReg
************************************************************************/
void A9101_WriteReg(Uint16 addr, Uint8 dataByte)
{
	XBYTE[addr] = dataByte;
	_nop_(); _nop_(); _nop_();
	_nop_(); _nop_(); _nop_();
}
/*********************************************************************
** SetRFCH
*********************************************************************/
void SetRFCH(Uint8 ch)
{
	A9101_WriteReg(PLL1_REG, ch);
}
/************************************************************************
**  A9101_ReadReg
************************************************************************/
Uint8 A9101_ReadReg(Uint16 addr)
{
    return (XBYTE[addr]);
}

/************************************************************************
**  Reset Command
************************************************************************/
void ResetCMD(Uint8 cmd)
{
    A9101_WriteReg(MODE_REG, cmd);
	Delay10us(1);
}

/************************************************************************
**  Strobe Command
************************************************************************/
void StrobeCMD(Uint8 cmd)
{
    A9101_WriteReg(MODEC1_REG, cmd);
	Delay10us(1);
}

/************************************************************************
**  A9101_WriteFIFO
************************************************************************/
void A9101_WriteFIFO(void)
{
	Uint16 i;	
	ResetCMD(TXPOINT_RST);	//TX FIFO address pointer reset
	for(i=0; i<64; i++)
	{
		A9101_WriteReg((i + TX_FIFO), PN9_Tab[i]);
	}

}

/************************************************************************
**  RxPacket
************************************************************************/
void RxPacket(void)
{
    Uint16 i;
    Uint8 recv;
    Uint8 tmp;

	RxCnt++;

	ResetCMD(RXPOINT_RST);	//RX FIFO address pointer reset

	for(i=0; i<64; i++)
	{	
		tmpbuf[i] = A9101_ReadReg(i + RX_FIFO);
		recv = tmpbuf[i];
		tmp = recv ^ PN9_Tab[i];
		if(tmp!=0)
		{
			P1_0 	= ~P1_0;
			Err_ByteCnt++;
			Err_BitCnt += (BitCount_Tab[tmp>>4] + BitCount_Tab[tmp & 0x0F]);
		}
    }

}



/*********************************************************************
** PM1
*********************************************************************/
void PM1(void)
{
	//all pin as input and pull high
	P0OE	= 0x00;
	P0PUN	= 0x00;
	P0		= 0xFF;
	P0WUN	= 0xFF;

	P1OE	= 0x00;
	P1PUN	= 0x00;
    P1		= 0xFF;
	P1WUN	= 0xFF;

	P3OE	= 0x00;
	P3PUN	= 0x00;
	P3		= 0xFF;
	P3WUN	= 0xFF;

	A9101_WriteReg(EXT5_REG, 0x08);
	A9101_WriteReg(PWRCTL_REG, 0xD0);
	StrobeCMD(CMD_SLEEP);
	PCONE = 0x20;
	PCON = 0x02;
	_nop_(); _nop_();
	while(PCON & 0x02);
}

/*********************************************************************
** PM2
*********************************************************************/
void PM2(void)
{
	//all pin as input and pull high
	P0OE	= 0x00;
	P0PUN	= 0x00;
	P0		= 0xFF;
	P0WUN	= 0xFF;

	P1OE	= 0x00;
	P1PUN	= 0x00;
    P1		= 0xFF;
	P1WUN	= 0xFF;

	P3OE	= 0x00;
	P3PUN	= 0x00;
	P3		= 0xFF;
	P3WUN	= 0xFF;

	A9101_WriteReg(EXT5_REG, 0x08);
	A9101_WriteReg(PWRCTL_REG, 0xB0);
	StrobeCMD(CMD_SLEEP);
	PCONE = 0x09;
	PCON = 0x02;
	_nop_(); _nop_();
	while(PCON & 0x02);
}


/*********************************************************************
** PM3
*********************************************************************/
void PM3(void)
{
	//all pin as input and pull high
	P0OE	= 0x00;
	P0PUN	= 0x00;
	P0		= 0xFF;
	P0WUN	= 0xFF;

	P1OE	= 0x00;
	P1PUN	= 0x00;
    P1		= 0xFF;
	P1WUN	= 0xFF;

	P3OE	= 0x00;
	P3PUN	= 0x00;
	P3		= 0xFF;
	P3WUN	= 0xFF;

	A9101_WriteReg(EXT5_REG, 0x08);
	A9101_WriteReg(PWRCTL_REG, 0xA8);
	StrobeCMD(CMD_SLEEP);
	PCONE = 0x09;
	PCON = 0x02;
	_nop_(); _nop_();
	while(PCON & 0x02);
}

/*********************************************************************
** RCOSC_Cal
*********************************************************************/
void RCOSC_Cal(void)
{
	Uint16 tmp;
	Uint16 TGNUM=1000;	//crystal=16.384M
		
	StrobeCMD(CMD_STBY);

	//A9101_WriteReg(CKO_REG, 0x50);	//ROSC
	//A9101_WriteReg(CKO_REG, 0x40);	//WCK

	A9101_WriteReg(RCOSC7_REG, (TGNUM/256));	//TGNUM
	A9101_WriteReg(RCOSC8_REG, TGNUM);			//TGNUM
	
	A9101_WriteReg(RCOSC3_REG, A9101Config[RCOSC3_REG - CONFIG] | 0x04);	//enable RC Oscillator

	while(1)
	{
		A9101_WriteReg(RCOSC4_REG, A9101Config[RCOSC4_REG - CONFIG] | 0x01);//RC OSC Calibration
		do{
        	tmp = A9101_ReadReg(RCOSC4_REG);
    	}while(tmp & 0x01);

		tmp = (A9101_ReadReg(RCOSC4_REG) & 0xF0);	//read NUMLH[11:8]
		tmp <<= 4;
		tmp = tmp + A9101_ReadReg(RCOSC5_REG);		//read NUMLH[7:0]
		
		if((tmp > (TGNUM-10)) && (tmp < (TGNUM+10)))
		{
			break;
		}
	}
}

/*********************************************************************
** RTC_Enable
*********************************************************************/
void RTC_Enable(void)
{
	IOSEL |= 0x10;	//enable RTCIOS
	P3OE  &= 0x3F;
	P3PUN &= 0x3F;

	//A9101_WriteReg(CKO_REG, 0x50);	//ROSC
	//A9101_WriteReg(CKO_REG, 0x40);	//WCK

	A9101_WriteReg(RCOSC3_REG, A9101Config[RCOSC3_REG - CONFIG] | 0x04);	//enable RC Oscillator
	A9101_WriteReg(RTC_REG, 0x01);		//RTCE=1
	A9101_WriteReg(RCOSC4_REG, 0x80);	//RCOT=4

	//delay for RTC stabilized
	//...
	//...
}

/*********************************************************************
** TMR_enable
*********************************************************************/
void TMR_enable(void)
{
	RCOSC_Cal();		//internal RC OSC CAL
//	RTC_Enable();		//use external RC OSC


	//TMR Period = (1 / (2 ^ (7-TMRCKS[2:0]) ) ) * (TMRINV + 1)
	//ex : TMRCKS=0, TMRINV=0
	//     TMR Period = (1 / (2 ^ (7-0) ) ) * (0+1) = 1/128 = 7.8125ms

	A9101_WriteReg(TMRINV_REG, 0);	//TMRINV=0
 
//	A9101_WriteReg(TMRCTL_REG, 0xE1);	//TMRCKS=128Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE3);	//TMRCKS=64Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE5);	//TMRCKS=32Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE7);	//TMRCKS=16Hz
	//A9101_WriteReg(TMRCTL_REG, 0xE9);	//TMRCKS=8Hz
	//A9101_WriteReg(TMRCTL_REG, 0xEB);	//TMRCKS=4Hz
	//A9101_WriteReg(TMRCTL_REG, 0xED);	//TMRCKS=2Hz
	A9101_WriteReg(TMRCTL_REG, 0xEF);	//TMRCKS=1Hz
	
	A9101_WriteReg(INTSW_REG, 0x01);	//enable TMR INT
	ERFINT=1;	//enable RF interrupt

//	PM1();
//	PM2();
//	PM3();
	
	//defined process by user
	//...
	//...
}
/************************************************************************
**  SAR_ADC_12B
************************************************************************/
void SAR_ADC_12B(void)
{
	Uint16 MVADC_value;
//	Uint8 tmp;

	StrobeCMD(CMD_STBY);

	//I/O set
	IOSEL |= 0x20;		//enable ADCIOS

	IOSEL |= 0x00;		//ADC0=P3.2
	P3OE &= ~0x04;	    //P3.2 as input

//	IOSEL |= 0x40;		//ADC1=P3.3
//	P3OE &= ~0x08;	    //P3.3 as input
//
//	IOSEL |= 0x80;		//ADC2=P3.4
//	P3OE &= ~0x10;	    //P3.4 as input
//
//	IOSEL |= 0xC0;		//ADC3=P3.5
//	P3OE &= ~0x20;	    //P3.4 as input


	A9101_WriteReg(ADC_REG, A9101Config[ADC_REG - CONFIG] | 0x04);	//set XADS

	EA=1; 	//enable interrupt
  	ERFINT=1; //enable RF interrupt
  	A9101_WriteReg(INTSW_REG, Enable_RFINT_ADC);
 
	while ( 1 )
	{
  
	  	A9101_WriteReg(ADCCTL_REG, 0x6F);		//CKS=3, MVS=7, single mode, ADCE=1

//		do{
//			tmp = A9101ReadReg(ADCCTL_REG);  	//wait ADC conversion complete
//		}while(tmp & 0x01);
	
	    RF_FLAG=1;
	    while(RF_FLAG);    //wait ADC RFISR conversion complete
	
	    //read MVADC
		MVADC_value = (A9101_ReadReg(ADCAVG1_REG) & 0xF0);
		MVADC_value <<= 4;
		MVADC_value = MVADC_value + A9101_ReadReg(ADCAVG2_REG);
	}
}
/************************************************************************
**  RSSI_measurement
************************************************************************/
void RSSI_measurement(void)
{
  	Uint8 tmp;

  	StrobeCMD(CMD_STBY);

//	A9101_WriteReg(GIO1_REG, 0x04);	//GPIO1=Fsync
//	A9101_WriteReg(GIO2_REG, 0x00);	//GPIO2=WTR

	A9101_WriteReg(MODEC2_REG, A9101Config[MODEC2_REG - CONFIG] | 0x40);	//ARSSI=1

  	EA=1; 	//enable interrupt
	ERFINT=1; //enable RF interrupt
  	A9101_WriteReg(INTSW_REG, Enable_RFINT_FSTNC); //enable FSYNC interrupt

  	StrobeCMD(CMD_RX);

	RF_FLAG=1;
  	while(RF_FLAG)   //Stay in RX mode until receiving ID code(sync ok)
  	{
		;
//    	tmp = A9101_ReadReg (RSSI_REG); //read RSSI value(environment RSSI)
  	}

  	tmp = A9101_ReadReg (RSSI_REG);   //read RSSI value(wanted signal RSSI)
	RSSI_value=tmp;
}
/************************************************************************
**  WOR_enable_by_preamble
************************************************************************/
void WOR_enable_by_preamble(void)
{

	RCOSC_Cal();    //internal RC OSC CAL
//	RTC_Enable();   //use external RC OSC

//	A9101_WriteReg(CKO_REG, 0x08);		//CKO=RCK
//	A9101_WriteReg(GPIO1_REG, 0x0C);	//GPIO1=PMDO
//	A9101_WriteReg(GPIO1_REG, 0x10);	//GPIO1=TWWS
//	A9101_WriteReg(GPIO2_REG, 0x00);	//GPIO2=WTR

	//setup WOR Sleep time and Rx time
	//WOR Active Period = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
	//WOR Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
	A9101_WriteReg(RCOSC1_REG, 0x7F);	//WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
	A9101_WriteReg(RCOSC2_REG, 0x00);
	A9101_WriteReg(TCODE_REG, 0x80 );	//WOR_AC[8:0]=256 : WOR Active Period ~ 7.84ms
	
	//WOR_HOLD : WOR hold RX setting when preamble/sync/carrier detected ok.
	//      [0]: No hold. [1]: Hold RX.
	//WOR_SEL: TWWS=1 setting.
	//      [0]:RX valid packet.  [1]: preamble/sync/carrier  detected ok.
	//WOR_RST: Shall be set to [1].
	A9101_WriteReg(WOR_REG, 0x3A);		//by preamble, WOR_SEL=1, WOR_HOLD=1, WOR_RST=1

	StrobeCMD(CMD_STBY);
	EA=1;	
	ERFINT=1; //enable RF interrupt
	A9101_WriteReg(INTSW_REG, Enable_RFINT_TWWS);  //enable TWWS INT
	
	A9101_WriteReg(MODEC2_REG, A9101Config[MODEC2_REG - CONFIG] | 0x08);	//enable WOR function

	PM2();
	
	//WOR_SEL=1 : preamble/sync/carrier detected ok -> TWWS go high -> wake up MCU
	//defined process by user
	//...
	//...
	//StrobeCMD(CMD_STBY);  //TWWS reset by strobe command.
}
/************************************************************************
**  WOR_enable_by_sync
************************************************************************/
void WOR_enable_by_sync(void)
{

	RCOSC_Cal();    //internal RC OSC CAL
//	RTC_Enable();   //use external RC OSC

//	A9101_WriteReg(CKO_REG, 0x08);		//CKO=RCK
//	A9101_WriteReg(GPIO1_REG, 0x04);	//GPIO1=Fsync
//	A9101_WriteReg(GPIO1_REG, 0x10);	//GPIO1=TWWS
//	A9101_WriteReg(GPIO2_REG, 0x00);	//GPIO2=WTR

	//setup WOR Sleep time and Rx time
	//WOR Active Period = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
	//WOR Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
	A9101_WriteReg(RCOSC1_REG, 0x7F);	//WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
	A9101_WriteReg(RCOSC2_REG, 0x00);
	A9101_WriteReg(TCODE_REG, 0x80 );	//WOR_AC[8:0]=256 : WOR Active Period ~ 7.84ms
	
	//WOR_HOLD : WOR hold RX setting when preamble/sync/carrier detected ok.
	//      [0]: No hold. [1]: Hold RX.
	//WOR_SEL: TWWS=1 setting.
	//      [0]:RX valid packet.  [1]: preamble/sync/carrier  detected ok.
	//WOR_RST: Shall be set to [1].
	A9101_WriteReg(WOR_REG, 0x38);		//by Fsync, WOR_SEL=1, WOR_HOLD=1, WOR_RST=1

	StrobeCMD(CMD_STBY);
	EA=1;	
	ERFINT=1; //enable RF interrupt
	A9101_WriteReg(INTSW_REG, Enable_RFINT_TWWS);  //enable TWWS INT
	
	A9101_WriteReg(MODEC2_REG, A9101Config[MODEC2_REG - CONFIG] | 0x08);	//enable WOR function

	PM2();
	
	//WOR_SEL=1 : preamble/sync/carrier detected ok -> TWWS go high -> wake up MCU
	//defined process by user
	//...
	//...
	//StrobeCMD(CMD_STBY);  //TWWS reset by strobe command.
}
/************************************************************************
**  WOR_enable_by_carrier
************************************************************************/
void WOR_enable_by_carrier(void)
{

	RCOSC_Cal();    //internal RC OSC CAL
//	RTC_Enable();   //use external RC OSC

//	A9101_WriteReg(CKO_REG, 0x08);		//CKO=RCK
//	A9101_WriteReg(GPIO1_REG, 0x08);	//GPIO1=CD
//	A9101_WriteReg(GPIO1_REG, 0x10);	//GPIO1=TWWS
//	A9101_WriteReg(GPIO2_REG, 0x00);	//GPIO2=WTR

	//setup WOR Sleep time and Rx time
	//WOR Active Period = (WOR_AC[8:0]+1) x (1/32768), (30.5us ~ 15.6ms).
	//WOR Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
	A9101_WriteReg(RCOSC1_REG, 0x7F);	//WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
	A9101_WriteReg(RCOSC2_REG, 0x00);
	A9101_WriteReg(TCODE_REG, 0x80 );	//WOR_AC[8:0]=256 : WOR Active Period ~ 7.84ms
	
	//WOR_HOLD : WOR hold RX setting when preamble/sync/carrier detected ok.
	//      [0]: No hold. [1]: Hold RX.
	//WOR_SEL: TWWS=1 setting.
	//      [0]:RX valid packet.  [1]: preamble/sync/carrier  detected ok.
	//WOR_RST: Shall be set to [1].
	A9101_WriteReg(WOR_REG, 0x3C);		//by carrier, WOR_SEL=1, WOR_HOLD=1, WOR_RST=1

	StrobeCMD(CMD_STBY);
	EA=1;	
	ERFINT=1; //enable RF interrupt
	A9101_WriteReg(INTSW_REG, Enable_RFINT_TWWS);  //enable TWWS INT
	
	A9101_WriteReg(RSSI_REG, 0x6E);		//RTH 
	A9101_WriteReg(MODEC2_REG, A9101Config[MODEC2_REG - CONFIG] | 0x48);	//enable WOR function & ARSSI=1

	PM2();
	
	//WOR_SEL=1 : preamble/sync/carrier detected ok -> TWWS go high -> wake up MCU
	//defined process by user
	//...
	//...
	//StrobeCMD(CMD_STBY);  //TWWS reset by strobe command.
}
/************************************************************************
**  WOT_enable
************************************************************************/
void WOT_enable(void)
{
	RCOSC_Cal();    //internal RC OSC CAL
//	RTC_Enable();   //use external RC OSC
	
//	A9101_WriteReg(CKO_REG, 0x00);		//CKO=DCK
//	A9101_WriteReg(GPIO1_REG, 0x04);	//GPIO1=Fsync
//	A9101_WriteReg(GPIO1_REG, 0x10);	//GPIO1=TWWS
//	A9101_WriteReg(GPIO2_REG, 0x00);	//GPIO2=WTR

	//setup WOT Sleep time
	//WOT Sleep  Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
	A9101_WriteReg(RCOSC1_REG, 0x7F);	//WOR_SL[9:0]=127 : WOR Sleep  Period ~ 1s
	A9101_WriteReg(RCOSC2_REG, 0x00);
	
	//WOR_RST: Shall be set to [1].
	A9101_WriteReg (WOR_REG, 0x21); //select WOT=1 function, WOR_RST=1
	
	StrobeCMD(CMD_STBY);
	A9101_WriteFIFO();  //write data to TX FIFO
	
	EA=1;
	ERFINT=1; //enable RF interrupt
	A9101_WriteReg(INTSW_REG, Enable_RFINT_TWWS);  //enable TWWS INT
		
	A9101_WriteReg(MODEC2_REG, A9101Config[MODEC2_REG - CONFIG] | 0x08);	//enable WOR function			
	
	PM2();
	
	//defined process by user
	//...
	//...
	//A9101_WriteReg(MODEC2_REG, A9101Config[MODEC2_REG - CONFIG]);			//disable WOT function
	//StrobeCMD(CMD_STBY);  //TWWS reset by strobe command.
}
/************************************************************************
**  TWOR_enable
************************************************************************/
void TWOR_enable ( void )
{
	RCOSC_Cal();    //internal RC OSC CAL
//	RTC_Enable();   //use external RC OSC
	
//	A9101_WriteReg(GPIO1_REG, 0x10);	//GPIO1=TWWS
//	A9101_WriteReg(GPIO2_REG, 0x10);	//GPIO1=TWWS
	
	//WOR_AC Period = (WOR_AC[8:0]+1) x (1/4096), (244us ~ 125ms).
	//WOR_SL Period = (WOR_SL[9:0]+1) x (32/4096), (7.8ms ~ 7.99s).
	//Note : TWOR function which enables this device to output a periodic square wave from GPIO to a MCU
	//                       _____     _____     _____
	//    TWWS signal : _____|   |_____|   |_____|   |
	//                    1s    1s    1s    1s    1s    1s
	//
	//    TWWS interrupt are activated by a rising edge.
	
	A9101_WriteReg(RCOSC1_REG, 0x7F);		//WOR_SL[9:0]=127 : WOR_SL Period ~ 1s
	A9101_WriteReg(RCOSC2_REG, 0x19);
	A9101_WriteReg(TCODE_REG, 0xC0);		//WOR_AC[8:0]=409 : WOR_AC Period ~ 0.1s
	
	EA=1;
	ERFINT=1; //enable RF interrupt
	A9101_WriteReg (INTSW_REG, Enable_RFINT_TWWS);  //enable TWWS INT
	
//	A9101_WriteReg(RCOSC3_REG, A9101Config[RCOSC3_REG - CONFIG] | 0x05);	//enable TWOR function by WOR_AC
	A9101_WriteReg(RCOSC3_REG, A9101Config[RCOSC3_REG - CONFIG] | 0x07);	//enable TWOR function by WOR_SL	

	PM2();
	
	//TWWS go high -> wake up MCU
	//defined process by user
	//...
	//...
	//A9101_WriteReg(RCOSC3_REG, A9101Config[RCOSC3_REG - CONFIG]);  		//disable TWOR function
}
/************************************************************************
**  A9101_Config
************************************************************************/
void A9101_Config(void)
{
	Uint16 i;

	memcpy(A9101Config, 0, sizeof(A9101Config));

#ifdef DR_2048Kbps 

	memcpy(A9101Config, A9101Config_2M, sizeof(A9101Config));

#else

	memcpy(A9101Config, A9101Config_512K, sizeof(A9101Config));

#endif

    for(i=0; i<sizeof(A9101Config); i++)
        A9101_WriteReg(i + CONFIG, A9101Config[i]);

}
/*********************************************************************
** Auto_Resend
*********************************************************************/
void Auto_Resend(void)
{
    StrobeCMD(CMD_STBY);

	A9101_WriteReg(GIO1_REG, 0x38);		//GPIO1=VPOAK
	A9101_WriteReg(GIO2_REG, 0x00);		//GPIO2=WTR
   
  	A9101_WriteReg(FCR_REG, 0x09);		//ARC=2, EAR=1, enable Auto Resend
    A9101_WriteReg(ARD_REG, 0x05);		//ARD=5
    //ACK format = Preamble + ID = 8bytes
    //data rate=2Mbps, Transmission Time=(1/2M)*8*8=0.12ms
    //so set ARD=5, RX time=5*200us=1ms

    while(1)
    {
        A9101_WriteFIFO();
        StrobeCMD(CMD_TX);      //entry TX mode
        Delay10us(1);
        while(GIO2);            //wait transmit completed

        //check VPOAK
        if(GIO1)
        {
            //ACK ok
        }
        else
        {
            //NO ACK
        }

        StrobeCMD(CMD_STBY);    //clear VPOAK(GIO1 signal)

        Delay10ms(1);
    }
}

/*********************************************************************
** Auto_ACK
*********************************************************************/
void Auto_ACK(void)
{
    StrobeCMD(CMD_STBY);
    
	A9101_WriteReg(GIO1_REG, 0x04);		//GPIO1=FSYNC
	A9101_WriteReg(GIO2_REG, 0x00);		//GPIO2=WTR

    A9101_WriteReg(FCR_REG, 0x02);  //enable Auto ACK

    while(1)
    {	
        StrobeCMD(CMD_RX);      //entry RX mode
        Delay10us(1);           
        while(GIO2);            //wait receive completed

        RxPacket();
    }
}
/************************************************************************
**  FCC
************************************************************************/
void FCC(void)
{
	Uint16 i;

	IOSEL |= 0x08;
 	EA=1;
    ERFINT=1;	
    A9101_WriteReg(INTSW_REG, Enable_RFINT_WTR);

	A9101_WriteReg(FIFO1_REG, 0x7F);	//128Byte
	ResetCMD(TXPOINT_RST);				//TX FIFO address pointer reset
	for(i=0; i<64; i++)
	{
		A9101_WriteReg((i + TX_FIFO), PN9_Tab[i]);
	}
	for(i=64; i<128; i++)
	{
		A9101_WriteReg((i + TX_FIFO), PN9_Tab[i-64]);
	}

	SetRFCH(60);	//915.361M

	StrobeCMD(CMD_PLL);

    while(1)
    {  	
		RF_FLAG=1;
    	StrobeCMD(CMD_TX);
		while(RF_FLAG==1);  		  

		Delay10us(1);
    }

	//Direct Tx or Rx
	//A9101_WriteReg(MODEC2_REG, 0x02);	//direct mode
	//StrobeCMD(CMD_TX);
	//StrobeCMD(CMD_RX);
	//while(1);

}
/************************************************************************
**  Err_State
************************************************************************/
void Err_State(void)
{
    //ERR display
    //Error Proc...
    //...
    
    //		/* Watchdog reset */
//		EIE|=0x20;
//	
//		CKCON=0x00;	
//		CKCON=0x40;
//		CKCON=0x80;
//		CKCON=0xC0;
//		
//		EA=1;
//		
//		TA=0xAA;//timed access register procedure
//		TA=0x55;
//		WDCON=0x02;
//		/* Watchdog reset End*/

    while(1);
}